
class Model {
	LPD3DXMESH Mesh;
	D3DMATERIAL8* MeshMaterials;
	LPDIRECT3DTEXTURE8* MeshTextures;
	DWORD NumMaterials;
	D3DXMATRIX *matWorld;
	BOOL selfLumin;
	
	LPDIRECT3DDEVICE8 D3DDevice;

public:
	Model();

	Model(LPD3DXMESH, D3DMATERIAL8*, LPDIRECT3DTEXTURE8*, DWORD);

	~Model();

	VOID setMatrix(D3DXMATRIX *);
	VOID setMatrix(D3DXMATRIX);

	VOID setDevice(LPDIRECT3DDEVICE8);

	VOID setSelfLumin(BOOL);

	VOID render();
};

Model::Model() {
	matWorld = NULL;
	D3DDevice = NULL;
	Mesh = NULL;
	selfLumin = FALSE;
	MeshMaterials = NULL;
	MeshTextures = NULL;
	NumMaterials = NULL;
}

Model::Model(LPD3DXMESH iMesh, D3DMATERIAL8* iMeshMaterials, LPDIRECT3DTEXTURE8* iMeshTextures, DWORD iNumMaterials){
	matWorld = NULL;
	D3DDevice = NULL;
	Mesh = iMesh;
	selfLumin = FALSE;
	MeshMaterials = iMeshMaterials;
	MeshTextures = iMeshTextures;
	NumMaterials = iNumMaterials;
}

Model::~Model() {
	if (MeshMaterials != NULL)
		delete[] MeshMaterials;
	if (MeshTextures != NULL) {
		for (DWORD i = 0; i < NumMaterials; i++) {
			if (MeshTextures[i])
				MeshTextures[i]->Release();
		}
		delete[] MeshTextures;
	}
	if (Mesh != NULL)
		Mesh->Release();
	D3DDevice = NULL;
}

VOID Model::setMatrix(D3DXMATRIX matUse) {
	matWorld = new D3DXMATRIX(matUse);
}

VOID Model::setMatrix(D3DXMATRIX *matUse) {
	matWorld = matUse;
}

VOID Model::setDevice(LPDIRECT3DDEVICE8 iD3DDevice) {
	D3DDevice = iD3DDevice;
}

VOID Model::setSelfLumin(BOOL sl) {
	selfLumin = sl;
}

VOID Model::render() {
	D3DDevice->SetTransform(D3DTS_WORLD, matWorld);
	D3DDevice->SetRenderState(D3DRS_LIGHTING, !selfLumin);
	for (DWORD i = 0; i < NumMaterials; i++) {
		D3DDevice->SetMaterial(&MeshMaterials[i]);
		D3DDevice->SetTexture(0, MeshTextures[i]);

		Mesh->DrawSubset(i);
	}
}
