#include <windows.h>
#include <mmsystem.h>
#include <dinput.h>
#include <stdio.h>
#include <d3dx8.h>
#include "D3DFont.h"
#include "Model.h"

LPDIRECT3D8 g_pD3D = NULL;
LPDIRECT3DDEVICE8 g_pd3dDevice = NULL;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

inline DWORD FtoDW( FLOAT f ) { return *((DWORD*)&f); }

/*
HRESULT LoadMesh(char *file, LPDIRECT3DDEVICE8 g_pd3dDevice, LPD3DXMESH *g_pMesh, D3DMATERIAL8 **g_pMeshMaterials, LPDIRECT3DTEXTURE8** g_pMeshTextures, DWORD *g_dwNumMaterials) {
	LPD3DXBUFFER pD3DXMtrlBuffer;

	if (FAILED(D3DXLoadMeshFromX(
		file,
		D3DXMESH_SYSTEMMEM,
		g_pd3dDevice, NULL,
		&pD3DXMtrlBuffer,
		g_dwNumMaterials,
		g_pMesh)))
		return E_FAIL;

	D3DXMATERIAL *d3dxMaterials = (D3DXMATERIAL *)pD3DXMtrlBuffer->GetBufferPointer();
	(*g_pMeshMaterials) = new D3DMATERIAL8[(*g_dwNumMaterials)];
	(*g_pMeshTextures) = new LPDIRECT3DTEXTURE8[(*g_dwNumMaterials)];

	for (DWORD i = 0; i < (*g_dwNumMaterials); i ++) {
		(*g_pMeshMaterials)[i] = d3dxMaterials[i].MatD3D;
		g_pMeshMaterials[i]->Ambient = g_pMeshMaterials[i]->Diffuse;
		if (FAILED(D3DXCreateTextureFromFile(g_pd3dDevice,
                                               d3dxMaterials[i].pTextureFilename, 
                                               g_pMeshTextures[i]))) {
			(*g_pMeshTextures)[i] = NULL;
		}
	}

	pD3DXMtrlBuffer->Release();

	return S_OK;
}
*/
HRESULT LoadMesh(char *file, LPDIRECT3DDEVICE8 g_pd3dDevice, Model **model) {
	LPD3DXMESH g_pMesh;
	D3DMATERIAL8* g_pMeshMaterials;
	LPDIRECT3DTEXTURE8* g_pMeshTextures;
	DWORD g_dwNumMaterials;

	LPD3DXBUFFER pD3DXMtrlBuffer;

	if (FAILED(D3DXLoadMeshFromX(
		file,
		D3DXMESH_SYSTEMMEM,
		g_pd3dDevice, NULL,
		&pD3DXMtrlBuffer,
		&g_dwNumMaterials,
		&g_pMesh)))
		return E_FAIL;

	D3DXMATERIAL *d3dxMaterials = (D3DXMATERIAL *)pD3DXMtrlBuffer->GetBufferPointer();
	g_pMeshMaterials = new D3DMATERIAL8[g_dwNumMaterials];
	g_pMeshTextures = new LPDIRECT3DTEXTURE8[g_dwNumMaterials];

	for (DWORD i = 0; i < g_dwNumMaterials; i ++) {
		g_pMeshMaterials[i] = d3dxMaterials[i].MatD3D;
		g_pMeshMaterials[i].Ambient = g_pMeshMaterials[i].Diffuse;
		if (FAILED(D3DXCreateTextureFromFile(g_pd3dDevice,
                                               d3dxMaterials[i].pTextureFilename, 
                                               &g_pMeshTextures[i]))) {
			g_pMeshTextures[i] = NULL;
		}
	}

	pD3DXMtrlBuffer->Release();

	(*model) = new Model(g_pMesh, g_pMeshMaterials, g_pMeshTextures, g_dwNumMaterials);
	(*model)->setDevice(g_pd3dDevice);
	return S_OK;
}

VOID SetupMatrices() {
	D3DXMATRIX matView;
	D3DXMatrixLookAtLH(&matView,
		&D3DXVECTOR3(0.0f, 3.0f, -5.0f),
		&D3DXVECTOR3(0.0f, 0.0f,  0.0f),
		&D3DXVECTOR3(0.0f, 1.0f,  0.0f));
	g_pd3dDevice->SetTransform(D3DTS_VIEW, &matView);
	
	D3DXMATRIX matProj;
//	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 3.0f/4.0f, 0.000001f, 1.0f);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 3.0f/4.0f, 0.01f, 10000.0f);
//	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 3.0f/4.0f, 0.0001f, 1000.0f);
//	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 3.0f/4.0f, 0.0f, 1000.0f);

	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &matProj);
}

VOID SetupLights() {
	D3DXVECTOR3 vecDir;
	D3DLIGHT8 light;
	ZeroMemory(&light, sizeof(D3DLIGHT8));
	light.Type = D3DLIGHT_DIRECTIONAL;
	light.Diffuse.r = 1.0f;
	light.Diffuse.g = 1.0f;
	light.Diffuse.b = 0.8f;
	light.Specular.r = 0.0f;
	light.Specular.g = 0.0f;
	light.Specular.b = 0.0f;
	light.Ambient.r = 0.0f;
	light.Ambient.g = 0.0f;
	light.Ambient.b = 0.0f;
	vecDir = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
	D3DXVec3Normalize((D3DXVECTOR3 *)&light.Direction, &vecDir);
	light.Range = 1000.0f;

	g_pd3dDevice->SetLight(0, &light);
	g_pd3dDevice->LightEnable(0, TRUE);

	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_pd3dDevice->SetRenderState(D3DRS_AMBIENT, 0x00808080);

	ZeroMemory(&light, sizeof(D3DLIGHT8));
	light.Type = D3DLIGHT_POINT;
	light.Diffuse.r = 0.0f;
	light.Diffuse.g = 0.0f;
	light.Diffuse.b = 1.0f;
	light.Specular.r = 0.0f;
	light.Specular.g = 0.0f;
	light.Specular.b = 0.0f;
	light.Ambient.r = 0.0f;
	light.Ambient.g = 0.0f;
	light.Ambient.b = 0.0f;
	vecDir = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	light.Position = vecDir;
	light.Range = 400.0f;
	light.Attenuation0 = 0.0f;
	light.Attenuation1 = 0.02f;
	light.Attenuation2 = 0.00f;

	g_pd3dDevice->SetLight(1, &light);
	g_pd3dDevice->LightEnable(1, TRUE);

	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_pd3dDevice->SetRenderState(D3DRS_AMBIENT, 0x00808080);

}

typedef Model *LPMODEL;

class Shine {
	struct POINTVERTEX
	{
		D3DXVECTOR3 pos;
		D3DCOLOR    color;
	};

	enum {D3DFVF_POINTVERTEX = D3DFVF_XYZ|D3DFVF_DIFFUSE};

	LPDIRECT3DTEXTURE8 ShineTexture;
	LPDIRECT3DVERTEXBUFFER8 D3DVB;
	LPDIRECT3DDEVICE8 D3DDevice;

	D3DXCOLOR *ShineColor;
	DWORD Size, MinSize, a0, a1, a2;

public:
	Shine();
	Shine(LPDIRECT3DTEXTURE8);

	VOID setDevice(LPDIRECT3DDEVICE8);

	VOID setColor(D3DXCOLOR);
	VOID setColor(D3DXCOLOR *);
	VOID setSize(FLOAT, FLOAT);
	VOID setScale(FLOAT, FLOAT, FLOAT);

	VOID render();
};

Shine::Shine() {
	ShineTexture = NULL;
	D3DDevice = NULL;
	D3DVB = NULL;

	ShineColor = NULL;
	Size = MinSize = a0 = a1 = a2 = 0;
}

Shine::Shine(LPDIRECT3DTEXTURE8 lt) {
	ShineTexture = lt;
	D3DDevice = NULL;
	D3DVB = NULL;

	ShineColor = NULL;
	Size = MinSize = a0 = a1 = a2 = 0;
}

VOID Shine::setDevice(LPDIRECT3DDEVICE8 id3d) {
	D3DDevice = id3d;

	D3DDevice->CreateVertexBuffer(
		1*sizeof(POINTVERTEX),
		0,
		D3DFVF_POINTVERTEX,
		D3DPOOL_DEFAULT, &D3DVB);

	POINTVERTEX *pVertices;
	D3DVB->Lock(0, 0, (BYTE **)&pVertices, 0);

	pVertices[0].pos.x = 0.0f;
	pVertices[0].pos.y = 0.0f;
	pVertices[0].pos.z = 0.0f;
	pVertices[0].color = (*ShineColor);

	D3DVB->Unlock();
}

VOID Shine::setColor(D3DXCOLOR ic) {
	ShineColor = new D3DXCOLOR(ic);
}

VOID Shine::setColor(D3DXCOLOR *ic) {
	ShineColor = ic;
}

VOID Shine::setSize(FLOAT is, FLOAT imin) {
	Size = *((DWORD *)(&is));
	MinSize = *((DWORD *)(&imin));
}

VOID Shine::setScale(FLOAT ia0, FLOAT ia1, FLOAT ia2) {
	a0 = *((DWORD *)(&ia0));
	a1 = *((DWORD *)(&ia1));
	a2 = *((DWORD *)(&ia2));
}

VOID Shine::render() {
	D3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
	D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

    D3DDevice->SetTexture(0, ShineTexture);

    D3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    D3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    D3DDevice->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
    D3DDevice->SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);

    D3DDevice->SetRenderState(D3DRS_POINTSPRITEENABLE, TRUE);
    D3DDevice->SetRenderState(D3DRS_POINTSCALEENABLE,  TRUE);

    D3DDevice->SetRenderState(D3DRS_POINTSIZE,     Size);
    D3DDevice->SetRenderState(D3DRS_POINTSIZE_MIN, MinSize);
//	D3DDevice->SetRenderState(D3DRS_POINTSIZE_MAX, FtoDW(300.0f));
    D3DDevice->SetRenderState(D3DRS_POINTSCALE_A,  a0);
    D3DDevice->SetRenderState(D3DRS_POINTSCALE_B,  a1);
    D3DDevice->SetRenderState(D3DRS_POINTSCALE_C,  a2);

    D3DDevice->SetStreamSource(0, D3DVB, sizeof(POINTVERTEX));
    D3DDevice->SetVertexShader(D3DFVF_POINTVERTEX);

	D3DDevice->DrawPrimitive(D3DPT_POINTLIST, 0, 2);
	D3DDevice->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
	D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
}

class Obj {
	FLOAT wx, wy, wz;	//poloha vzhledem k pozorovateli
	FLOAT v2;			//ctverec vzdalenosti od pozorovatele;
public:
	DOUBLE *x, *y, *z;	//absolutni poloha
	DOUBLE vx, vy, vz;	//absolutni rychlost
	DOUBLE *a, *b, *g;	//natoceni
	DOUBLE va, vb, vg;	//rychlost rotace

	DOUBLE *cx, *cy, *cz;//stred blizke gravitace

	FLOAT v;			//vzdalenost od pozorovatele

	FLOAT size;
	FLOAT mass;

	LPMODEL model;
	BOOL rendered;
	FLOAT v2max0;		//maximalni vzdalenost pro zobrazeni ve scene
	FLOAT v2max1;		//maximalni vzdalenost pro zobrazeni mimo scenu

	Obj();
	VOID Obj::render();

	VOID render0();
	VOID render1();
	VOID render2();

	VOID setWorld(DOUBLE, DOUBLE, DOUBLE);
	VOID setPosition(DOUBLE, DOUBLE, DOUBLE);
	VOID setPosition(DOUBLE *, DOUBLE *, DOUBLE *);
	VOID setRotation(DOUBLE, DOUBLE, DOUBLE);
	VOID setRotation(DOUBLE *, DOUBLE *, DOUBLE *);

	VOID setv2max0(FLOAT);
	VOID setv2max1(FLOAT);
};

Obj::Obj() {
	x = y = z = NULL;
	wx = wy = wz = 0;
	a = b = g = NULL;
	va = vb = vg = 0;
	cx = cy = cz = NULL;

	v2 = 0;
	v2max0 = v2max1 = 0;
	size = 0;
	mass = 0;
	model = NULL;
	rendered = FALSE;
}

VOID Obj::render() {
	D3DXMATRIX mat1;
	D3DXMatrixTranslation(&mat1, wx, wy, wz);
	model->setMatrix(&mat1);
	model->render();
}

VOID Obj::render0() {
	if (!rendered) {
		rendered = TRUE;
		render();
	}
}

VOID Obj::render1() {
	if ((!rendered)&&(v2 > v2max0)) {
		rendered = TRUE;
		render();
	}
}

VOID Obj::render2() {
	if ((!rendered)&&(v2 > v2max1)) {
		rendered = TRUE;
//		render();	Nebude se renderovat
	}
}

VOID Obj::setWorld(DOUBLE ix, DOUBLE iy, DOUBLE iz) {
	wx = (FLOAT)(*x - ix);
	wy = (FLOAT)(*y - iy);
	wz = (FLOAT)(*z - iz);
	v2 = wx*wx + wy*wy + wz*wz;
	v = sqrtf(v2);
	rendered = FALSE;
}

VOID Obj::setPosition(DOUBLE ix, DOUBLE iy, DOUBLE iz) {
	x = new DOUBLE(ix);
	y = new DOUBLE(iy);
	z = new DOUBLE(iz);
}

VOID Obj::setPosition(DOUBLE *ix, DOUBLE *iy, DOUBLE *iz) {
	x = ix;
	y = iy;
	z = iz;
}

VOID Obj::setRotation(DOUBLE ia, DOUBLE ib, DOUBLE ig) {
	a = new DOUBLE(ia);
	b = new DOUBLE(ib);
	g = new DOUBLE(ig);
}

VOID Obj::setRotation(DOUBLE *ia, DOUBLE *ib, DOUBLE *ig) {
	a = ia;
	b = ib;
	g = ig;
}

VOID Obj::setv2max0(FLOAT vmax) {
	v2max0 = vmax*vmax;
}

VOID Obj::setv2max1(FLOAT vmax) {
	v2max1 = vmax*vmax;
}

CD3DFont* m_pFont;

LPMODEL ship;
Shine *shine;

Obj stardock;
Obj borg;

Obj earth, sun, moon;

FLOAT v;
//FLOAT x=-400, y=800, z=800;
FLOAT x=-160, y=100, z=100;

D3DXVECTOR3 t, b, l;
D3DXMATRIX matShip;
D3DXMATRIX matView;

D3DXMATRIX W1, W2;

HRESULT InitGeometry() {

	LPDIRECT3DTEXTURE8 tex;
	if (FAILED(D3DXCreateTextureFromFile(g_pd3dDevice, "particle1.bmp", &tex)))
		return E_FAIL;
	shine = new Shine(tex);
	shine->setColor(D3DCOLOR_XRGB(255, 255, 255));
	shine->setSize(1.0f, 0.0f);
	shine->setScale(0.0f, 0.0f, 0.1f);
	shine->setDevice(g_pd3dDevice);

	if (FAILED(LoadMesh("ship.x", g_pd3dDevice, &ship))) 
		return E_FAIL;

	D3DXMatrixIdentity(&matShip);
	l.x = matShip._11;
	l.y = matShip._12;
	l.z = matShip._13;

	b.x = matShip._21;
	b.y = matShip._22;
	b.z = matShip._23;

	t.x = matShip._31;
	t.y = matShip._32;
	t.z = matShip._33;

	if (FAILED(LoadMesh("stardock.x", g_pd3dDevice, &stardock.model))) 
		return E_FAIL;
	stardock.setv2max0(500.0f);
	stardock.setv2max1(100000.0f);
	stardock.setPosition(0.0f, 0.0f, 0.0f);
	stardock.size = 500.0f;

	if (FAILED(LoadMesh("borg.x", g_pd3dDevice, &borg.model))) 
		return E_FAIL;
	borg.setv2max0(750.0f);
	borg.setv2max1(1000000.0f);
	borg.size = 500.0f;
	borg.setPosition(1000.0f, 500.0f, 100.0f);

	if (FAILED(LoadMesh("earth.x", g_pd3dDevice, &earth.model))) 
		return E_FAIL;
	earth.setPosition(700000.0f, 0.0f, 0.0f);
	earth.setv2max0(0.0f);
	earth.setv2max1(1000000000000.0f);

	if (FAILED(LoadMesh("sun.x", g_pd3dDevice, &sun.model))) 
		return E_FAIL;
	sun.setPosition(-14960000000.0f, 0.0f, 0.0f);
//	sun.setPosition(-3960000000.0f, 0.0f, 0.0f);	//visual
	sun.setv2max0(0.0f);
	sun.setv2max1(1000000000000.0f);

	if (FAILED(LoadMesh("moon.x", g_pd3dDevice, &moon.model))) 
		return E_FAIL;
	moon.setPosition(0.0f, -38440000.0f, 0.0f);
//	moon.setPosition(0.0f, -9440000.0f, 0.0f); //visual
	moon.setv2max0(0.0f);
	moon.setv2max1(1000000000000.0f);

	SetupMatrices();
	SetupLights();

	return S_OK;
}

HRESULT InitD3D(HWND hWnd) {
	if (NULL == (g_pD3D = Direct3DCreate8(D3D_SDK_VERSION)) )
		return E_FAIL;

	D3DDISPLAYMODE d3ddm;
	if (FAILED(g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
		return E_FAIL;

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory(&d3dpp, sizeof(d3dpp));
	d3dpp.Windowed = TRUE;

	d3dpp.Windowed = FALSE;
	d3dpp.BackBufferWidth = 640;
	d3dpp.BackBufferHeight = 480;

	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.BackBufferFormat = D3DFMT_R5G6B5;
//	d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
//	d3dpp.AutoDepthStencilFormat = D3DFMT_D24S8;
//	d3dpp.AutoDepthStencilFormat = D3DFMT_D32;
//	d3dpp.AutoDepthStencilFormat = D3DFMT_D24X8;

//	d3dpp.MultiSampleType = D3DMULTISAMPLE_2_SAMPLES;


	if (FAILED(g_pD3D->CreateDevice(
		D3DADAPTER_DEFAULT,
		D3DDEVTYPE_HAL,
		hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp, &g_pd3dDevice)))
		return E_FAIL;
/*
	LPDIRECT3DSURFACE8 pBackBuffer = NULL;
    g_pd3dDevice->GetRenderTarget(&pBackBuffer);

	g_pd3dDevice->CreateDepthStencilSurface(1024, 768, D3DFMT_D32, D3DMULTISAMPLE_NONE, &g_pzBuffer);
	g_pd3dDevice->SetRenderTarget(pBackBuffer, g_pzBuffer);
	pBackBuffer->Release();
	g_pzBuffer->Release();
*/
    m_pFont = new CD3DFont( TEXT("Arial"), 12, D3DFONT_BOLD );
    m_pFont->InitDeviceObjects( g_pd3dDevice );
    m_pFont->RestoreDeviceObjects();

//	g_pd3dDevice->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_USEW);
//	g_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);

	return S_OK;
}

LPDIRECTINPUT8 lpdi = NULL;
LPDIRECTINPUTDEVICE8  lpdiKeyboard = NULL;
LPDIRECTINPUTDEVICE8  lpdiMouse = NULL;

HRESULT InitDInput( HWND hwnd ) {
	if (FAILED(DirectInput8Create(GetModuleHandle(NULL), DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID **) &lpdi, NULL)))
		return E_FAIL;

	if (FAILED(lpdi->CreateDevice(GUID_SysKeyboard, &lpdiKeyboard, NULL))) 
		return E_FAIL;
	lpdiKeyboard->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
//	lpdiKeyboard->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
	lpdiKeyboard->SetDataFormat(&c_dfDIKeyboard);

	if (FAILED(lpdiKeyboard->Acquire()))
		return E_FAIL;

	if (FAILED(lpdi->CreateDevice(GUID_SysMouse, &lpdiMouse, NULL))) 
		return E_FAIL;

	lpdiMouse->SetDataFormat(&c_dfDIMouse);
//	lpdiMouse->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
//	lpdiMouse->SetCooperativeLevel(hwnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
	lpdiMouse->SetCooperativeLevel(hwnd, DISCL_EXCLUSIVE | DISCL_FOREGROUND);

	if (FAILED(lpdiMouse->Acquire()))
		return E_FAIL;
	return S_OK;
}


void CleanDInput( ) {
	if (lpdiKeyboard != NULL) {
		lpdiKeyboard->Unacquire();
		lpdiKeyboard->Release();
	}
	if (lpdiMouse != NULL) {
		lpdiMouse->Unacquire();
		lpdiMouse->Release();
	}
	if (lpdi != NULL) lpdi->Release();
}


VOID Cleanup() {
	if (g_pd3dDevice != NULL) 
		g_pd3dDevice->Release();
	if (g_pD3D != NULL)
		g_pD3D->Release();
	if (m_pFont != NULL) m_pFont->DeleteDeviceObjects();
}

char KeyBuffer[256];
DIMOUSESTATE MouseState;

BOOL konec = FALSE;

void Drive() {

	FLOAT alpha, gamma, beta;
    lpdiMouse->GetDeviceState(sizeof(MouseState),(LPVOID)&MouseState);
    gamma = (FLOAT)MouseState.lY*0.005f;
    alpha = (FLOAT)MouseState.lX*0.005f;
    beta = (FLOAT)MouseState.lZ*0.0005f;

    lpdiKeyboard->GetDeviceState(sizeof(KeyBuffer),(LPVOID)&KeyBuffer); 
    if (KeyBuffer[DIK_UP]&0x80) v += 1.0f; 
    if (KeyBuffer[DIK_DOWN]&0x80) v -= 1.0f; 
    if (KeyBuffer[DIK_SPACE]&0x80) v = 0.0f;
    if (KeyBuffer[DIK_W]&0x80) v += 100.0f; 
    if (KeyBuffer[DIK_S]&0x80) v -= 100.0f; 
    if (KeyBuffer[DIK_A]&0x80) beta = 0.1f;
    if (KeyBuffer[DIK_D]&0x80) beta = -0.1f;
    if (KeyBuffer[DIK_X]&0x80) v *= 2.0f;
    if (KeyBuffer[DIK_ESCAPE]&0x80) konec = true;

	if (MouseState.rgbButtons[1])
		v -= 0.01f;
//		v -= 1.0f;

	if (MouseState.rgbButtons[0])
		v += 0.01f;
//		v += 1.0f;


	D3DXMATRIX matW1, matW2;

	D3DXMatrixRotationAxis(&matW1, &t, beta);
	D3DXMatrixMultiply(&matShip, &matShip, &matW1);
	D3DXMatrixRotationAxis(&matW1, &b, alpha);
	D3DXMatrixMultiply(&matShip, &matShip, &matW1);
	D3DXMatrixRotationAxis(&matW1, &l, gamma);
	D3DXMatrixMultiply(&matShip, &matShip, &matW1);

	l.x = matShip._11;
	l.y = matShip._12;
	l.z = matShip._13;

	b.x = matShip._21;
	b.y = matShip._22;
	b.z = matShip._23;

	t.x = matShip._31;
	t.y = matShip._32;
	t.z = matShip._33;

}

D3DXMATRIX matW1, matW2;

VOID Camera() {
/*
	D3DXMatrixLookAtLH(&matView,
		&D3DXVECTOR3(-250.0f, 100.0f, 100.0f),
		&D3DXVECTOR3(x, y, z),
		&D3DXVECTOR3(0.0f, 1.0f,  0.0f));

	D3DXMatrixLookAtLH(&matView,
		&D3DXVECTOR3(-250.0f-x, +100.0f-y, +100.0f-z),
		&D3DXVECTOR3(0, 0, 0),
		&D3DXVECTOR3(0.0f, 1.0f,  0.0f));
	g_pd3dDevice->SetTransform(D3DTS_VIEW, &matView);
*/

	D3DXMatrixInverse(&matView, NULL, &matShip);
/*
	D3DXMatrixTranslation(&matW1, 0.0f, -1.0f, 0.5f);
//	D3DXMatrixTranslation(&matW1, -x+0.001f*t.x, -y+0.001f*t.y, -z+0.001f*t.z);
//	D3DXMatrixTranslation(&matW1, -x+0.0001f*t.x, -y+0.0001f*t.y, -z+0.0001f*t.z);
//	D3DXMatrixTranslation(&matW1, -x, -y, -z);
//	D3DXMatrixMultiply(&matW1, &matW1, &matView);
	D3DXMatrixMultiply(&matW1, &matView, &matW1);
	g_pd3dDevice->SetTransform(D3DTS_VIEW, &matW1);
*/
	g_pd3dDevice->SetTransform(D3DTS_VIEW, &matView);


}

VOID ShipMove() {
	x += t.x*v;
	y += t.y*v;
	z += t.z*v;

//	D3DXMatrixTranslation(&matW2, x, y, z);
//	D3DXMatrixMultiply(&matW1, &matShip, &matW2);

//	ship->setMatrix(&matW1);
	ship->setMatrix(&matShip);
}

DWORD numFPS = 0, cas = 0;
FLOAT fps = 0.0f;
char strFPS[20];

inline VOID FPS() {
    m_pFont->DrawText( 2,  0, D3DCOLOR_ARGB(255,255,255,0), strFPS );

	numFPS ++;

/*
	if (numFPS > 50)
	{
		fps = 50000.0f/(FLOAT)(timeGetTime() - cas);
		cas = timeGetTime();
		numFPS = 0;
		sprintf(strFPS, "FPS: %.1f", fps);
	}
*/

	if ((timeGetTime() - cas) >= 1000) {
		sprintf(strFPS, "FPS: %d", numFPS);
		numFPS = 0;
		cas = timeGetTime();
	}

}

VOID Render() {
	if (NULL == g_pd3dDevice)
		return;

	g_pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);
	g_pd3dDevice->BeginScene();

//	g_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	g_pd3dDevice->SetRenderState(D3DRS_DITHERENABLE, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
//	g_pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
/*
	g_pd3dDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);
	g_pd3dDevice->SetRenderState(D3DRS_FOGCOLOR, D3DCOLOR_XRGB(250, 150, 50));
	g_pd3dDevice->SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_LINEAR);
//	g_pd3dDevice->SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_EXP2);
//	g_pd3dDevice->SetRenderState(D3DRS_FOGDENSITY, FtoDW(0.01f));
//	g_pd3dDevice->SetRenderState(D3DRS_FOGVERTEXMODE, D3DFOG_LINEAR);
	g_pd3dDevice->SetRenderState(D3DRS_FOGSTART, FtoDW(100.0f));
	g_pd3dDevice->SetRenderState(D3DRS_FOGEND, FtoDW(500.0f));
	g_pd3dDevice->SetRenderState(D3DRS_RANGEFOGENABLE, TRUE);
*/
	ShipMove();
	Camera();

	stardock.setWorld(x, y, z);
	earth.setWorld(x, y, z);
	moon.setWorld(x, y, z);
	sun.setWorld(x, y, z);
	borg.setWorld(x, y, z);
	sun.model->setSelfLumin(TRUE);
	borg.model->setSelfLumin(TRUE);

	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);

	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MIPMAPLODBIAS, FtoDW(-2.5f));

	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, FALSE);

	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, 0.01f, 10000000.0f);
	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &matProj);

	sun.render2();
	moon.render2();
	earth.render2();
	stardock.render2();
	borg.render2();

	sun.render1();
	moon.render1();
	earth.render1();

	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);

//	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, stardock.v-stardock.size, stardock.v+stardock.size);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, 100.0f, stardock.v+stardock.size);
	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &matProj);
	stardock.render1();

//	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, borg.v-borg.size, borg.v+borg.size);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, 100.0f, borg.v+borg.size);
	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &matProj);
	borg.render1();

	g_pd3dDevice->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);
	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 4.0f/3.0f, 1.0f, 1000.0f);
	g_pd3dDevice->SetTransform(D3DTS_PROJECTION, &matProj);

//	earth.render0();
	stardock.render0();
	borg.render0();
//	ship->render();
//g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
	g_pd3dDevice->SetRenderState(D3DRS_FOGENABLE, FALSE);

//	g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );

//	g_pd3dDevice->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS, TRUE);
	D3DXMATRIX mat1;
	D3DXMatrixTranslation(&mat1, -x+0, -y+160, -z);
	g_pd3dDevice->SetTransform(D3DTS_WORLD, &mat1);
    
	shine->render();

	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_NONE);
	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_NONE);
	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_NONE);

	g_pd3dDevice->SetTextureStageState(0, D3DTSS_MIPMAPLODBIAS, FtoDW(0.0f));
//	g_pd3dDevice->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS, FALSE);
//	g_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);

//  g_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );

	FPS();

	g_pd3dDevice->EndScene();
	g_pd3dDevice->Present(NULL, NULL, NULL, NULL);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd) {
	static TCHAR AppName[] = TEXT("Test 2");
	HWND hwnd;
//	MSG msg;
	WNDCLASS WndClass = {
		CS_HREDRAW | CS_VREDRAW,
		WndProc,
		0,
		0,
		hInstance,
		LoadIcon (NULL, IDI_APPLICATION),
		LoadCursor(NULL, IDC_ARROW),
		(HBRUSH) GetStockObject(WHITE_BRUSH),
		NULL,
		AppName
	};

	if (!RegisterClass(&WndClass)) {
		MessageBox(NULL, TEXT("Tento program vyaduje Windows NT"), AppName, MB_ICONERROR);
		return 0;
	}

	hwnd = CreateWindow(
		AppName,
		TEXT("Test 1 : Creating a Device"),
		WS_OVERLAPPEDWINDOW,
		100, 100,
		600, 600,
		NULL, NULL,
		hInstance, NULL);

	if (SUCCEEDED(InitD3D(hwnd)) && SUCCEEDED(InitGeometry())) {
		ShowWindow(hwnd, nShowCmd);
		UpdateWindow(hwnd);

		if (SUCCEEDED(InitDInput(hwnd))) {
			while ( !konec ) {
					Drive();
					Render();
			}
/*
			BOOL gm = TRUE;
			while ( gm ) {
				if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ) {
					if( gm = GetMessage( &msg, NULL, 0, 0 ) ) {
						TranslateMessage( &msg );
						DispatchMessage( &msg );
					}
				}
				else {
					Drive();
					Render();
				}
			}
*/
	/*
			while (GetMessage (&msg, NULL, 0, 0)) {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
	*/
		} else
			MessageBox(NULL, TEXT("Nepodailo se inicializovat Direct Input"), AppName, NULL);

	} else
		MessageBox(NULL, TEXT("Nepodailo se inicializovat Direct 3D"), AppName, NULL);

	CleanDInput();
	Cleanup();
	return 0;
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
	switch (message) {
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	case WM_PAINT:
		Render();
		ValidateRect(hwnd, NULL);
		InvalidateRect(hwnd, NULL, FALSE);
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}